// UserExported.cpp: implementation of the UserExported class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "User.h"
#include "UserExported.h"
#include "UserRecord.h"
#include "Friend.h"
#include "BlockedUserRecord.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// These functions are not exportes and so not declard in .h file
int justAddAUser(UserRecord& m_UserRec, CString userName, CString password);
int justAddAFriend(CFriend& m_FriendRec, CString userName, CString friendName);
int justAddABlockedUser(CBlockedUserRecord& m_BlockedUserRec, CString userName, CString blockedUserName);
int changePassword(CString oldPassword, CString newPassword);
bool isAUser(CString userName);


// This function checks for the existence of a user inside the user database
// Returns TRUE if user found and password matches
// FALSE otherwise
BOOL isValid(CString userName, CString password)
{
	UserRecord m_UserRec;
	CString u, p;

	if(m_UserRec.IsOpen())
		m_UserRec.Close();

	m_UserRec.Open();

	// Check whether it's an empty recordset
	if(m_UserRec.IsBOF())
		return FALSE;

	// Go to first user
	m_UserRec.MoveFirst();

	// Traverse thru the record set
	while(!m_UserRec.IsEOF())
	{
		if((userName == m_UserRec.m_UserName) && (password == m_UserRec.m_Password))
			return TRUE;

		m_UserRec.MoveNext();
	}

	m_UserRec.Close();

	return FALSE;
}


// Adds a user to the User list
// Returns: 0 - Success
//			1 - User already exists
//			2 - User creation failed due to some other reason
int addUser(CString userName, CString password)
{
	UserRecord m_UserRec;

	if(m_UserRec.IsOpen())
	{
		//database.Close();
		m_UserRec.Close();
	}

	m_UserRec.Open();

	// Check whether it's an empty recordset
	if(m_UserRec.IsBOF())
		return(justAddAUser(m_UserRec, userName, password));

	// Traverse thru the record set
	while(!m_UserRec.IsEOF())
	{
		if(userName == m_UserRec.m_UserName)
			return 1;

		m_UserRec.MoveNext();
	}

	// If the user is not found, add it to the end
	return(justAddAUser(m_UserRec, userName, password));
	
	// Close the record set
	//database.Close();
	m_UserRec.Close();

	return 0;
}


// Adds a single record to the User table
int justAddAUser(UserRecord& m_UserRec, CString userName, CString password)
{
	int ret;

	try
	{
		// Create space for a new record and put values into it
		m_UserRec.AddNew();
		m_UserRec.m_UserName = userName;
		m_UserRec.m_Password = password;

		// Commit the record
		if(!m_UserRec.Update())
			ret = 2;
		else
			ret = 0;
	}
	catch(CDBException)
	{
		ret = 2;
	}

	// Close the record set
	//m_UserRec.m_pDatabase->Close();
	m_UserRec.Close();

	return ret;
}

// Add a friend's name to the friend list
int addAFriend(CString userName, CString friendName)
{
	CFriend m_FriendRec;

	// First check whether the friend is a registered user
	if(!isAUser(friendName))
		return 3;

	if(m_FriendRec.IsOpen())
	{
		//database.Close();
		m_FriendRec.Close();
	}

	m_FriendRec.Open();

	// Check whether it's an empty recordset
	if(m_FriendRec.IsBOF())
		return(justAddAFriend(m_FriendRec, userName, friendName));

	// Traverse thru the record set
	while(!m_FriendRec.IsEOF())
	{
		// If this friend name already exists
		if((userName == m_FriendRec.m_UserName) && (friendName == m_FriendRec.m_FriendName))
			return 1;

		m_FriendRec.MoveNext();
	}

	// If the user is not found, add it to the end
	return(justAddAFriend(m_FriendRec, userName, friendName));
	
	// Close the record set
	m_FriendRec.Close();

	return 0;
}


// Adds a single record to the Friends table
int justAddAFriend(CFriend& m_FriendRec, CString userName, CString friendName)
{
	int ret;

	try
	{
		// Create space for a new record and put values into it
		m_FriendRec.AddNew();
		m_FriendRec.m_UserName = userName;
		m_FriendRec.m_FriendName = friendName;

		// Commit the record
		if(!m_FriendRec.Update())
			ret = 2;
		else
			ret = 0;
	}
	catch(CDBException)
	{
		ret = 2;
	}

	// Close the record set
	m_FriendRec.Close();

	return ret;
}

bool isAUser(CString userName)
{
	UserRecord m_UserRec;

	if(m_UserRec.IsOpen())
	{
		m_UserRec.Close();
	}

	m_UserRec.Open();

	// Check whether it's an empty recordset
	if(m_UserRec.IsBOF())
		return false;

	// Traverse thru the record set
	while(!m_UserRec.IsEOF())
	{
		if(userName == m_UserRec.m_UserName)
			return true;

		m_UserRec.MoveNext();
	}

	// Close the record set
	m_UserRec.Close();

	return false;
}

int changePassword(CString userName, CString oldPassword, CString newPassword)
{
	// If the old password supplied by the user is incorrect
	if(!isValid(userName, oldPassword))
		return 1;

	// If old password is ok, try changing the password
	UserRecord m_UserRec;

	if(m_UserRec.IsOpen())
	{
		m_UserRec.Close();
	}

	CString qry = "Select * from User where UserName = '" + userName + "'";
	int ret = m_UserRec.Open(AFX_DB_USE_DEFAULT_TYPE, qry);

	if(!ret)
	{
		m_UserRec.Close();
		return 2;
	}

	// Update the record
	try
	{
		m_UserRec.Edit();
		m_UserRec.m_Password = newPassword;
		m_UserRec.Update();
	}
	catch(CException *e)
	{
		// Close the record set
		m_UserRec.Close();

		return 3;
	}

	// Close the record set
	m_UserRec.Close();

	return 0;
}

// Delete a friend's name from the friends list
int deleteAFriend(CString userName, CString friendName)
{
	CFriend m_FriendRec;

	try
	{

	if(m_FriendRec.IsOpen())
	{
		m_FriendRec.Close();
	}

	CString qry = "Select * from Friends where UserName = '" + userName + "' and FriendName = '" + friendName + "'";
	int ret = m_FriendRec.Open(AFX_DB_USE_DEFAULT_TYPE, qry);

	if(!ret)
		return 1;

	// Delete the friend
	m_FriendRec.Delete();

	// Close the record set
	m_FriendRec.Close();
	}
	catch(CException *e)
	{
		TRACE("Exception");
		return 2;
	}

	return 0;
}


// Add a friend's name to the friend list
int addABlockedUser(CString userName, CString blockedUserName)
{
	CBlockedUserRecord m_BlockedUserRec;

	// First check whether the friend is a registered user
	if(!isAUser(blockedUserName))
		return 3;

	if(m_BlockedUserRec.IsOpen())
	{
		m_BlockedUserRec.Close();
	}

	m_BlockedUserRec.Open();

	// Check whether it's an empty recordset
	if(m_BlockedUserRec.IsBOF())
		return(justAddABlockedUser(m_BlockedUserRec, userName, blockedUserName));

	// Traverse thru the record set
	while(!m_BlockedUserRec.IsEOF())
	{
		// If this pair already exists
		if((userName == m_BlockedUserRec.m_UserName) && (blockedUserName == m_BlockedUserRec.m_BlockedUserName))
			return 1;

		m_BlockedUserRec.MoveNext();
	}

	// If the user is not found, add it to the end
	return(justAddABlockedUser(m_BlockedUserRec, userName, blockedUserName));
	
	// Close the record set
	m_BlockedUserRec.Close();

	return 0;
}



// Adds a single record to the Friends table
int justAddABlockedUser(CBlockedUserRecord& m_BlockedUserRec, CString userName, CString blockedUserName)
{
	int ret;

	try
	{
		// Create space for a new record and put values into it
		m_BlockedUserRec.AddNew();
		m_BlockedUserRec.m_UserName = userName;
		m_BlockedUserRec.m_BlockedUserName = blockedUserName;

		// Commit the record
		if(!m_BlockedUserRec.Update())
			ret = 2;
		else
			ret = 0;
	}
	catch(CDBException)
	{
		ret = 2;
	}

	// Close the record set
	m_BlockedUserRec.Close();

	return ret;
}

// Delete a friend's name from the friends list
int deleteABlockedUser(CString userName, CString blockedUserName)
{
	CBlockedUserRecord m_BlockedRec;

	try
	{

	if(m_BlockedRec.IsOpen())
	{
		m_BlockedRec.Close();
	}

	CString qry = "Select * from BlockedUser where UserName = '" + userName + "' and BlockedUserName = '" + blockedUserName + "'";
	int ret = m_BlockedRec.Open(AFX_DB_USE_DEFAULT_TYPE, qry);

	if(!ret)
		return 1;

	// Delete the friend
	m_BlockedRec.Delete();

	// Close the record set
	m_BlockedRec.Close();
	}
	catch(CException *e)
	{
		TRACE("Exception");
		return 2;
	}

	return 0;
}
